from __future__ import annotations

from pathlib import Path
from shutil import which

import pytest
from ase.build import bulk
from numpy.testing import assert_allclose

from quacc.recipes.espresso.dos import dos_flow, projwfc_flow
from quacc.utils.files import copy_decompress_files

pytestmark = pytest.mark.skipif(
    which("pw.x") is None or which("dos.x") is None, reason="QE not installed"
)

DATA_DIR = Path(__file__).parent / "data"


def test_dos_flow(tmp_path, monkeypatch):
    monkeypatch.chdir(tmp_path)
    monkeypatch.setenv("OMP_NUM_THREADS", "1")

    copy_decompress_files(DATA_DIR, ["Si.upf.gz"], tmp_path)
    atoms = bulk("Si")
    input_data = {
        "control": {"calculation": "scf", "pseudo_dir": tmp_path},
        "electrons": {"mixing_mode": "TF", "mixing_beta": 0.7, "conv_thr": 1.0e-6},
    }

    pseudopotentials = {"Si": "Si.upf"}

    job_params = {
        "static_job": {"input_data": input_data, "pseudopotentials": pseudopotentials},
        "non_scf_job": {"kspacing": 0.05},
    }

    output = dos_flow(atoms, job_params=job_params)
    assert_allclose(
        output["static_job"]["atoms"].get_positions(),
        atoms.get_positions(),
        atol=1.0e-4,
    )
    assert (
        output["static_job"]["parameters"]["input_data"]["control"]["calculation"]
        == "scf"
    )
    assert (
        output["static_job"]["parameters"]["input_data"]["electrons"]["mixing_mode"]
        == "TF"
    )

    assert output["static_job"]["results"]["nbands"] == 8
    assert output["static_job"]["results"]["nspins"] == 1

    assert_allclose(
        output["non_scf_job"]["atoms"].get_positions(),
        atoms.get_positions(),
        atol=1.0e-4,
    )
    assert (
        output["non_scf_job"]["parameters"]["input_data"]["control"]["calculation"]
        == "nscf"
    )
    assert (
        output["non_scf_job"]["parameters"]["input_data"]["electrons"]["mixing_mode"]
        == "TF"
    )
    assert output["non_scf_job"]["results"]["nbands"] == 8
    assert output["non_scf_job"]["results"]["nspins"] == 1

    assert output["dos_job"]["results"]["dos_results"]["fermi"] == pytest.approx(6.772)


def test_projwfc_flow(tmp_path, monkeypatch):
    monkeypatch.chdir(tmp_path)
    monkeypatch.setenv("OMP_NUM_THREADS", "1")

    copy_decompress_files(DATA_DIR, ["Si.upf.gz"], tmp_path)
    atoms = bulk("Si")
    input_data = {
        "control": {"calculation": "scf", "pseudo_dir": tmp_path},
        "electrons": {"mixing_mode": "TF", "mixing_beta": 0.7, "conv_thr": 1.0e-6},
    }

    pseudopotentials = {"Si": "Si.upf"}

    job_params = {
        "static_job": {"input_data": input_data, "pseudopotentials": pseudopotentials},
        "non_scf_job": {"kspacing": 0.05},
    }

    output = projwfc_flow(atoms, job_params=job_params)
    assert_allclose(
        output["static_job"]["atoms"].get_positions(),
        atoms.get_positions(),
        atol=1.0e-4,
    )
    assert (
        output["static_job"]["parameters"]["input_data"]["control"]["calculation"]
        == "scf"
    )
    assert (
        output["static_job"]["parameters"]["input_data"]["electrons"]["mixing_mode"]
        == "TF"
    )

    assert output["static_job"]["results"]["nbands"] == 8
    assert output["static_job"]["results"]["nspins"] == 1

    assert_allclose(
        output["non_scf_job"]["atoms"].get_positions(),
        atoms.get_positions(),
        atol=1.0e-4,
    )
    assert (
        output["non_scf_job"]["parameters"]["input_data"]["control"]["calculation"]
        == "nscf"
    )
    assert (
        output["non_scf_job"]["parameters"]["input_data"]["electrons"]["mixing_mode"]
        == "TF"
    )
    assert output["non_scf_job"]["results"]["nbands"] == 8
    assert output["non_scf_job"]["results"]["nspins"] == 1
